'use strict';

const SLOTS_NAMES = [
  "default",
  "title",
  "footer"
];
const VISIBLE_KEY = "update:visible";

function isComponent(content) {
  return !Object.keys(content).some((key) => SLOTS_NAMES.includes(key));
}
function resolveSlots(instance, vm) {
  let { content } = instance;
  const scopedSlots = {};
  const children = [];
  if (content) {
    const resolve = instance.resolve;
    const h = vm.$createElement;
    const on = {
      close(...args) {
        vm.$data.visible = false;
        let value;
        if (args.length === 0)
          value = void 0;
        else if (args.length === 1)
          value = args[0];
        else
          value = args;
        resolve(value);
      }
    };
    if (isComponent(content)) {
      content = {
        default: {
          component: content,
          propsData: {}
        }
      };
    }
    Object.entries(content).forEach(([key, value]) => {
      if (!value.component) {
        value = {
          component: value
        };
      }
      const { propsData = {}, component } = value;
      if (component) {
        const vnode = h(component, { props: propsData, on });
        scopedSlots[key] = () => [vnode];
        if (key === "footer") {
          children.push(h("template", { slot: key }, [vnode]));
        }
      }
    });
  }
  return { scopedSlots, children };
}
function resolveOptions(instance, vm) {
  const { options: { open, opened, close, closed, ...props } } = instance;
  const on = {};
  const events = { open, opened, close, closed };
  Object.entries(events).filter(([, value]) => value).forEach(([key, value]) => on[key] = value);
  return {
    props: {
      ...props,
      visible: vm.$data.visible
    },
    on: {
      ...on,
      [VISIBLE_KEY](value) {
        vm.$data.visible = value;
      }
    }
  };
}

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
function Dialog(Vue) {
  return class Dialog {
    constructor(globalOptions, rootOptions) {
      __publicField(this, "globalOptions");
      __publicField(this, "rootOptions");
      __publicField(this, "options");
      __publicField(this, "vm");
      __publicField(this, "content");
      __publicField(this, "resolve");
      __publicField(this, "reject");
      this.globalOptions = globalOptions || {};
      this.rootOptions = rootOptions || {};
      this.options = {};
    }
    dialog(content, userOptions = {}) {
      if (!content || typeof content !== "object" || !Object.keys(content).length) {
        return Promise.reject(new Error("The content shown in 'el-dialog' component cannot be empty"));
      }
      this.options = Object.assign({}, this.globalOptions, userOptions);
      this.content = content;
      if (!this.vm) {
        this.vm = this.getInstance();
      }
      const vm = this.vm;
      document.body.appendChild(vm.$el);
      vm.$data.visible = this.options.visible;
      return new Promise((resolve, reject) => {
        this.resolve = resolve;
        this.reject = reject;
      });
    }
    getInstance() {
      const component = this.createComponent();
      const DialogCtor = Vue.extend(component);
      return new DialogCtor().$mount();
    }
    createComponent() {
      const instance = this;
      const options = {
        data() {
          return {
            visible: false
          };
        },
        render(h) {
          const { props, on } = resolveOptions(instance, this);
          const { scopedSlots, children } = resolveSlots(instance, this);
          return h(
            "ElDialog",
            {
              props,
              on,
              scopedSlots
            },
            children
          );
        }
      };
      return Object.assign({}, this.rootOptions, options);
    }
    destroy() {
      const { vm } = this;
      if (!vm)
        return true;
      if (vm.$el && vm.$el.parentNode) {
        vm.$el.parentNode.removeChild(vm.$el);
      }
      vm.$destroy();
      this.vm = null;
      return true;
    }
  };
}
Dialog.install = (Vue, globalOptions) => {
  const DialogClass = Dialog(Vue);
  const dialog = new DialogClass(globalOptions);
  Vue.prototype.$dialog = dialog.dialog.bind(dialog);
};

module.exports = Dialog;
